@@ -602,16 +602,21 @@ display_thread(void* vmarshal){
     if(ncv){
       int y;
       ncplane_cursor_yx(notcurses_stdplane_const(m->nc), &y, NULL);
+      bool pixeling = false;
+      if(notcurses_check_pixel_support(m->nc) >= 1){
+        pixeling = true;
+      }
       struct ncvisual_options vopts = {
         .x = NCALIGN_CENTER,
-        .blitter = NCBLIT_PIXEL,
-        .scaling = NCSCALE_SCALE_HIRES,
+        .blitter = pixeling ? NCBLIT_PIXEL : NCBLIT_3x2,
+        .scaling = pixeling ? NCSCALE_NONE : NCSCALE_SCALE_HIRES,
         .flags = NCVISUAL_OPTION_HORALIGNED,
       };
       struct ncplane* iplane = ncvisual_render(m->nc, ncv, &vopts);
       ncvisual_destroy(ncv);
       if(iplane){
-        ncplane_move_yx(iplane, y, 0);
+        int x = ncplane_x(iplane);
+        ncplane_move_yx(iplane, y, x);
         ncplane_scrollup_child(notcurses_stdplane(m->nc), iplane);
         notcurses_render(m->nc);
         ncplane_cursor_move_yx(notcurses_stdplane(m->nc),
@@ -664,7 +669,9 @@ ncneofetch(struct notcurses* nc){
   fi.hostname = notcurses_hostname();
   fi.username = notcurses_accountname();
   fetch_env_vars(nc, &fi);
-  fetch_x_props(&fi);
+  if(kern != NCNEO_XNU && kern != NCNEO_WINDOWS){
+    fetch_x_props(&fi);
+  }
   if(kern == NCNEO_LINUX){
     fetch_cpu_info(&fi);
   }else{
@@ -695,7 +702,8 @@ int main(int argc, char** argv){
     .flags = NCOPTION_SUPPRESS_BANNERS
              | NCOPTION_NO_ALTERNATE_SCREEN
              | NCOPTION_NO_CLEAR_BITMAPS
-             | NCOPTION_PRESERVE_CURSOR,
+             | NCOPTION_PRESERVE_CURSOR
+             | NCOPTION_DRAIN_INPUT,
   };
   if(argc > 2){
     usage(argv[0], stderr);
