@@ -337,9 +337,17 @@ display_logo(struct ncplane* n, const char* path){
 
 static void
 tinfo_debug_bitmaps(struct ncplane* n, const tinfo* ti, const char* indent){
-  ncplane_printf(n, "%sdefbg 0x%06lx %sconsidered transparent", indent,
-                 ti->bg_collides_default & 0xfffffful,
-                 (ti->bg_collides_default & 0x01000000) ? "" : "not ");
+  if(!(ti->bg_collides_default & 0x80000000)){
+    if(!(ti->bg_collides_default & 0x01000000)){
+      ncplane_printf(n, "%sdefault background 0x%06lx", indent,
+                    ti->bg_collides_default & 0xfffffful);
+    }else{
+      ncplane_printf(n, "%sdefault background 0x%06lx considered transparent", indent,
+                    ti->bg_collides_default & 0xfffffful);
+    }
+  }else{
+    ncplane_printf(n, "couldn't detect default background");
+  }
   finish_line(n);
   ncpixelimpl_e blit = notcurses_check_pixel_support(ncplane_notcurses(n));
   switch(blit){
@@ -414,6 +422,7 @@ tinfo_debug_styles(const notcurses* nc, struct ncplane* n, const char* indent){
   tinfo_debug_cap(n, "vid", notcurses_canopen_videos(nc));
   tinfo_debug_cap(n, "indn", get_escape(ti, ESCAPE_INDN));
   tinfo_debug_cap(n, "gpm", ti->gpmfd >= 0);
+  tinfo_debug_cap(n, "kbd", ti->kbdlevel > 0);
   finish_line(n);
 }
 
@@ -451,8 +460,11 @@ int main(int argc, const char** argv){
   int dimx;
   struct ncplane* stdn = notcurses_stddim_yx(nc, NULL, &dimx);
   if(dimx < 80){
+    ncplane_set_fg_rgb(stdn, 0xff5349);
+    ncplane_set_styles(stdn, NCSTYLE_BOLD);
+    ncplane_putstr(stdn, "This program requires at least 80 columns.\n");
+    notcurses_render(nc);
     notcurses_stop(nc);
-    fprintf(stderr, "This program requires at least 80 columns.\n");
     return EXIT_FAILURE;
   }
   ncplane_set_scrolling(stdn, true);
