@@ -9,9 +9,6 @@
 #include "visual-details.h"
 #include "notcurses/direct.h"
 #include "internal.h"
-#ifdef USE_READLINE
-#include <readline/readline.h>
-#endif
 
 // conform to the foreground and background channels of 'channels'
 static int
@@ -184,15 +181,19 @@ int ncdirect_cursor_disable(ncdirect* nc){
 }
 
 static int
-cursor_yx_get(ncdirect* n, int ttyfd, const char* u7, int* y, int* x){
+cursor_yx_get(ncdirect* n, const char* u7, int* y, int* x){
   struct inputctx* ictx = n->tcache.ictx;
   if(ncdirect_flush(n)){
     return -1;
   }
-  if(tty_emit(u7, ttyfd)){
-    return -1;
+  int fakey, fakex;
+  if(y == NULL){
+    y = &fakey;
   }
-  get_cursor_location(ictx, y, x);
+  if(x == NULL){
+    x = &fakex;
+  }
+  get_cursor_location(ictx, u7, y, x);
   loginfo("cursor at y=%d x=%d\n", *y, *x);
   return 0;
 }
@@ -210,7 +211,7 @@ int ncdirect_cursor_move_yx(ncdirect* n, int y, int x){
     if(hpa){
       return term_emit(tiparm(hpa, x), n->ttyfp, false);
     }else if(n->tcache.ttyfd >= 0 && u7){
-      if(cursor_yx_get(n, n->tcache.ttyfd, u7, &y, NULL)){
+      if(cursor_yx_get(n, u7, &y, NULL)){
         return -1;
       }
     }else{
@@ -220,7 +221,7 @@ int ncdirect_cursor_move_yx(ncdirect* n, int y, int x){
     if(!vpa){
       return term_emit(tiparm(vpa, y), n->ttyfp, false);
     }else if(n->tcache.ttyfd >= 0 && u7){
-      if(cursor_yx_get(n, n->tcache.ttyfd, u7, NULL, &x)){
+      if(cursor_yx_get(n, u7, NULL, &x)){
         return -1;
       }
     }else{
@@ -254,7 +255,7 @@ int ncdirect_cursor_move_yx(ncdirect* n, int y, int x){
 //    determine whether we're inverted.
 //  * if both dimensions changed, determine whether we're inverted by checking
 //    the change. the row ought have decreased; the column ought have increased.
-//  * move back to intiial position / pop cursor position
+//  * move back to initial position / pop cursor position
 static int
 detect_cursor_inversion(ncdirect* n, const char* u7, int rows, int cols, int* y, int* x){
   if(rows <= 1 || cols <= 1){ // FIXME can this be made to work in 1 dimension?
@@ -390,7 +391,7 @@ int ncdirect_cursor_yx(ncdirect* n, int* y, int* x){
   if(!x){
     x = &xval;
   }
-  ret = cursor_yx_get(n, n->tcache.ttyfd, u7, y, x);
+  ret = cursor_yx_get(n, u7, y, x);
   if(tcsetattr(n->tcache.ttyfd, TCSANOW, &oldtermios)){
     fprintf(stderr, "Couldn't restore terminal mode on %d (%s)\n",
             n->tcache.ttyfd, strerror(errno)); // don't return error for this
@@ -834,21 +835,17 @@ static int
 ncdirect_stop_minimal(void* vnc){
   ncdirect* nc = vnc;
   int ret = drop_signals(nc);
-  if(nc->initialized_readline){
-#ifdef USE_READLINE
-    rl_deprep_terminal();
-#endif
-  }
   fbuf f = {};
   if(fbuf_init_small(&f) == 0){
-    if(nc->tcache.pixel_shutdown){
-      ret |= nc->tcache.pixel_shutdown(&f);
-    }
     ret |= reset_term_attributes(&nc->tcache, &f);
     ret |= fbuf_finalize(&f, stdout);
   }
   if(nc->tcache.ttyfd >= 0){
-    ret |= tty_emit("\x1b[<u", nc->tcache.ttyfd);
+    if(nc->tcache.kbdlevel){
+      if(tty_emit(KKEYBOARD_POP, nc->tcache.ttyfd)){
+        ret = -1;
+      }
+    }
     const char* cnorm = get_escape(&nc->tcache, ESCAPE_CNORM);
     if(cnorm && tty_emit(cnorm, nc->tcache.ttyfd)){
       ret = -1;
@@ -864,7 +861,12 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
     outfp = stdout;
   }
   if(flags > (NCDIRECT_OPTION_DRAIN_INPUT << 1)){ // allow them through with warning
-    logwarn("Passed unsupported flags 0x%016jx\n", (uintmax_t)flags);
+    logwarn("Passed unsupported flags 0x%016" PRIx64 "\n", flags);
+  }
+  if(termtype){
+    if(putenv_term(termtype)){
+      return NULL;
+    }
   }
   ncdirect* ret = malloc(sizeof(ncdirect));
   if(ret == NULL){
@@ -904,7 +906,7 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
   int cursor_x = -1;
   if(interrogate_terminfo(&ret->tcache, termtype, ret->ttyfp, utf8, 1,
                           flags & NCDIRECT_OPTION_INHIBIT_CBREAK,
-                          0, &cursor_y, &cursor_x, &ret->stats, 0, 0,
+                          0, &cursor_y, &cursor_x, &ret->stats, 0, 0, 0, 0,
                           flags & NCDIRECT_OPTION_DRAIN_INPUT)){
     goto err;
   }
@@ -948,20 +950,146 @@ int ncdirect_stop(ncdirect* nc){
   return ret;
 }
 
+// our new input system is fundamentally incompatible with libreadline, so we
+// have to fake it ourselves. at least it saves us the dependency.
+//
+// if NCDIRECT_OPTION_INHIBIT_CBREAK is in play, we're not going to get the
+// text until cooked mode has had its way with it, and we are essentially
+// unable to do anything clever. text will be echoed, and there will be no
+// line-editing keybindings, save any implemented in the line discipline.
+//
+// otherwise, we control echo. whenever we emit output, get our position. if
+// we've changed line, assume the prompt has scrolled up, and account for
+// that. we return to the prompt, clear any affected lines, and reprint what
+// we have.
 char* ncdirect_readline(ncdirect* n, const char* prompt){
-#ifdef USE_READLINE
-  if(!n->initialized_readline){
-    rl_outstream = n->ttyfp;
-    rl_instream = stdin;
-    rl_prep_terminal(1); // 1 == read 8-bit input
-    n->initialized_readline = true;
+  const char* u7 = get_escape(&n->tcache, ESCAPE_U7);
+  if(!u7){ // we probably *can*, but it would be a pita; screw it
+    logerror("can't readline without u7\n");
+    return NULL;
   }
-  return readline(prompt);
-#else
-  (void)n;
-  (void)prompt;
+  if(fprintf(n->ttyfp, "%s", prompt) < 0){
+    return NULL;
+  }
+  int dimx = ncdirect_dim_x(n);
+  // FIXME what if we're reading from redirected input, not a terminal?
+  int y, xstart;
+  if(cursor_yx_get(n, u7, &y, &xstart)){
+    return NULL;
+  }
+  int tline = y;
+  int bline = y;
+  wchar_t* str;
+  int wspace = BUFSIZ / sizeof(*str);
+  if((str = malloc(wspace * sizeof(*str))) == NULL){
+    return NULL;
+  }
+  int wpos = 0;  // cursor location (single-dimensional)
+  int wused = 0; // number used
+  str[wused++] = L'\0';
+  ncinput ni;
+  uint32_t id;
+  int oldx = xstart;
+  while((id = ncdirect_getc_blocking(n, &ni)) != (uint32_t)-1){
+    if(id == NCKEY_EOF || id == NCKEY_ENTER){
+      if(id == NCKEY_ENTER){
+        if(fputc('\n', n->ttyfp) < 0){
+          free(str);
+          return NULL;
+        }
+      }else if(wused == 1){ // NCKEY_EOF without input returns NULL
+        free(str);
+        return NULL;
+      }
+      char* ustr = ncwcsrtombs(str);
+      free(str);
+      return ustr;
+    }else if(id == NCKEY_BACKSPACE){
+      if(wused > 1){
+        str[wused - 2] = L'\0';
+        --wused;
+      }
+      --wpos;
+    }else if(id == NCKEY_LEFT){
+      --wpos;
+    }else if(id == NCKEY_RIGHT){
+      ++wpos;
+    }else if(id == NCKEY_UP){
+      wpos -= dimx;
+    }else if(id == NCKEY_DOWN){
+      wpos += dimx;
+    }else if(id == 'a' && ni.ctrl){
+      wpos = 1;
+    }else if(id == 'e' && ni.ctrl){
+      wpos = wused - 1;
+    }else{
+      if(wspace - 1 < wused){
+        wspace += BUFSIZ;
+        wchar_t* tmp = realloc(str, wspace * sizeof(*str));
+        if(tmp == NULL){
+          free(str);
+          return NULL;
+        }
+        str = tmp;
+      }
+      if(wpos < wused - 1){
+        memmove(str + wpos + 1, str + wpos, (wused - wpos) * sizeof(*str));
+        str[wpos] = id;
+        ++wused;
+        ++wpos;
+      }else{
+        str[wused - 1] = id;
+        ++wused;
+        ++wpos;
+        str[wused - 1] = L'\0';
+      }
+      // FIXME check modifiers
+      int x;
+      if(cursor_yx_get(n, u7, &y, &x)){
+        break;
+      }
+      if(x < oldx){
+        oldx = x;
+        if(--tline < 0){
+          tline = 0;
+        }
+      }
+      if(y > bline){
+        bline = y;
+      }
+    }
+    if(wpos < 0){
+      wpos = 0;
+    }else if(wpos > wused - 1){
+      wpos = wused - 1;
+    }
+    // clear to end of line(s)
+    const char* el = get_escape(&n->tcache, ESCAPE_EL);
+    for(int i = bline ; i >= tline ; --i){
+      if(ncdirect_cursor_move_yx(n, i, i > tline ? 0 : xstart)){
+        break;
+      }
+      if(term_emit(el, n->ttyfp, false)){
+        break;
+      }
+    }
+    if(fprintf(n->ttyfp, "%ls", str) < 0){
+      break;
+    }
+    if(wpos != wused){
+      int linear = xstart + wpos;
+      int ylin = linear / dimx;
+      int xlin = linear % dimx;
+      if(ncdirect_cursor_move_yx(n, tline + ylin, xlin)){
+        break;
+      }
+    }
+    if(fflush(n->ttyfp)){
+      break;
+    }
+  }
+  free(str);
   return NULL;
-#endif
 }
 
 static inline int
@@ -1027,7 +1155,7 @@ int ncdirect_styles_off(ncdirect* n, unsigned stylebits){
   return ncdirect_off_styles(n, stylebits);
 }
 
-unsigned ncdirect_styles(ncdirect* n){
+unsigned ncdirect_styles(const ncdirect* n){
   return n->stylemask;
 }
 
