@@ -1166,7 +1166,7 @@ int kitty_draw(const tinfo* ti, const ncpile* p, sprixel* s, fbuf* f,
     animated = true;
   }
   int ret = s->glyph.used;
-  logdebug("dumping %zub for %u at %d %d\n", s->glyph.used, s->id, yoff, xoff);
+  logdebug("dumping %" PRIu64 "b for %u at %d %d\n", s->glyph.used, s->id, yoff, xoff);
   if(ret){
     if(fbuf_putn(f, s->glyph.buf, s->glyph.used) < 0){
       ret = -1;
@@ -1203,10 +1203,3 @@ int kitty_clear_all(fbuf* f){
   }
   return 0;
 }
-
-int kitty_shutdown(fbuf* f){
-  // FIXME need to close off any open kitty bitmap emission, or we will
-  // lock up the terminal
-  (void)f;
-  return 0;
-}
