@@ -52,9 +52,6 @@ int notcurses_leave_alternate_screen(notcurses* nc){
 int reset_term_attributes(const tinfo* ti, fbuf* f){
   int ret = 0;
   const char* esc;
-  if((esc = get_escape(ti, ESCAPE_RESTORECOLORS)) && fbuf_emit(f, esc)){
-    ret = -1;
-  }
   if((esc = get_escape(ti, ESCAPE_OP)) && fbuf_emit(f, esc)){
     ret = -1;
   }
@@ -83,13 +80,10 @@ notcurses_stop_minimal(void* vnc){
   // be sure to write the restoration sequences *prior* to running rmcup, as
   // they apply to the screen (alternate or otherwise) we're actually using.
   const char* esc;
-  // ECMA-48 suggests that we can interrupt an escape code with a NUL
-  // byte. if we leave an active escape open, it can lock up the terminal.
-  // we only want to do it when in the middle of a rasterization, though. FIXME
-  if(nc->tcache.pixel_shutdown){
-    ret |= nc->tcache.pixel_shutdown(f);
-  }
   ret |= mouse_disable(&nc->tcache, f);
+  if((esc = get_escape(&nc->tcache, ESCAPE_RESTORECOLORS)) && fbuf_emit(f, esc)){
+    ret = -1;
+  }
   ret |= reset_term_attributes(&nc->tcache, f);
   if((esc = get_escape(&nc->tcache, ESCAPE_RMKX)) && fbuf_emit(f, esc)){
     ret = -1;
@@ -106,8 +100,10 @@ notcurses_stop_minimal(void* vnc){
     if(nc->tcache.tpreserved){
       ret |= tcsetattr(nc->tcache.ttyfd, TCSAFLUSH, nc->tcache.tpreserved);
     }
-    if(tty_emit("\x1b[<u", nc->tcache.ttyfd)){
-      ret = -1;
+    if(nc->tcache.kbdlevel){
+      if(tty_emit(KKEYBOARD_POP, nc->tcache.ttyfd)){
+        ret = -1;
+      }
     }
     if((esc = get_escape(&nc->tcache, ESCAPE_RMCUP))){
       if(sprite_clear_all(&nc->tcache, f)){ // send this to f
@@ -263,40 +259,32 @@ int update_term_dimensions(int* rows, int* cols, tinfo* tcache, int margin_b){
     }
     return 0;
   }
-#ifndef __MINGW64__
-  struct winsize ws;
-  int i = ioctl(tcache->ttyfd, TIOCGWINSZ, &ws);
-  if(i < 0){
-    logerror("TIOCGWINSZ failed on %d (%s)\n", tcache->ttyfd, strerror(errno));
-    return -1;
-  }
-  if(ws.ws_row <= 0 || ws.ws_col <= 0){
-    logerror("Bogus return from TIOCGWINSZ on %d (%d/%d)\n",
-             tcache->ttyfd, ws.ws_row, ws.ws_col);
-    return -1;
-  }
-  int rowsafe;
+  int rowsafe, colsafe;
   if(rows == NULL){
     rows = &rowsafe;
   }
-  *rows = ws.ws_row;
-  if(cols){
-    *cols = ws.ws_col;
+  if(cols == NULL){
+    cols = &colsafe;
   }
+#ifndef __MINGW64__
+  struct winsize ws;
+  if(tiocgwinsz(tcache->ttyfd, &ws)){
+    return -1;
+  }
+  *rows = ws.ws_row;
+  *cols = ws.ws_col;
   if(tcache){
 #ifdef __linux__
     if(tcache->linux_fb_fd >= 0){
       get_linux_fb_pixelgeom(tcache, &tcache->pixy, &tcache->pixx);
     }else
 #endif
-    {
-      // we might have the pixel geometry from CSI14t, so don't override a
-      // valid earlier response with 0s from the ioctl. we do want to fire
-      // off a fresh CSI14t in this case, though FIXME.
-      if(ws.ws_ypixel){
-        tcache->pixy = ws.ws_ypixel;
-        tcache->pixx = ws.ws_xpixel;
-      }
+    // we might have the pixel geometry from CSI14t, so don't override a valid
+    // earlier response with 0s from the ioctl. we do want to fire off a fresh
+    // CSI14t in this case, though FIXME.
+    if(ws.ws_ypixel){
+      tcache->pixy = ws.ws_ypixel;
+      tcache->pixx = ws.ws_xpixel;
     }
     // update even if we didn't get values just now, because we need set
     // cellpix{y,x} up from an initial CSI14n, which set only pix{y,x}.
@@ -309,21 +297,14 @@ int update_term_dimensions(int* rows, int* cols, tinfo* tcache, int margin_b){
 #else
   CONSOLE_SCREEN_BUFFER_INFO csbi;
   if(GetConsoleScreenBufferInfo(tcache->outhandle, &csbi)){
-    if(cols){
-      *cols = csbi.srWindow.Right - csbi.srWindow.Left + 1;
-    }
-    if(rows){
-      *rows = csbi.srWindow.Bottom - csbi.srWindow.Top + 1;
-    }
+    *cols = csbi.srWindow.Right - csbi.srWindow.Left + 1;
+    *rows = csbi.srWindow.Bottom - csbi.srWindow.Top + 1;
   }else{
-    if(rows){
-      *rows = tcache->default_rows;
-    }
-    if(cols){
-      *cols = tcache->default_cols;
-    }
+    *rows = tcache->default_rows;
+    *cols = tcache->default_cols;
   }
 #endif
+  update_tinfo_geometry(tcache, *rows, *cols);
   if(tcache->sixel_maxy_pristine){
     int sixelrows = *rows - 1;
     // if the bottom margin is at least one row, we can draw into the last
@@ -441,8 +422,8 @@ make_ncpile(notcurses* nc, ncplane* n){
 // (as once more is n).
 ncplane* ncplane_new_internal(notcurses* nc, ncplane* n,
                               const ncplane_options* nopts){
-  if(nopts->flags >= (NCPLANE_OPTION_MARGINALIZED << 1u)){
-    logwarn("Provided unsupported flags %016jx\n", (uintmax_t)nopts->flags);
+  if(nopts->flags >= (NCPLANE_OPTION_FIXED << 1u)){
+    logwarn("Provided unsupported flags %016" PRIx64 "\n", nopts->flags);
   }
   if(nopts->flags & NCPLANE_OPTION_HORALIGNED || nopts->flags & NCPLANE_OPTION_VERALIGNED){
     if(n == NULL){
@@ -746,7 +727,7 @@ int ncplane_resize_internal(ncplane* n, int keepy, int keepx, int keepleny,
     logerror("Can't keep %d@%d cols from %d\n", keeplenx, keepx, cols);
     return -1;
   }
-  loginfo("%dx%d @ %d/%d → %d/%d @ %d/%d (keeping %dx%d from %d/%d)\n", rows, cols, n->absy, n->absx, ylen, xlen, n->absy + keepy + yoff, n->absx + keepx + xoff, keepleny, keeplenx, keepy, keepx);
+  loginfo("%dx%d @ %d/%d → %d/%d @ %d/%d (want %dx%d from %d/%d)\n", rows, cols, n->absy, n->absx, ylen, xlen, n->absy + keepy + yoff, n->absx + keepx + xoff, keepleny, keeplenx, keepy, keepx);
   if(n->absy == n->absy + keepy && n->absx == n->absx + keepx &&
       rows == ylen && cols == xlen){
     return 0;
@@ -876,13 +857,16 @@ int ncplane_destroy(ncplane* ncp){
     if( (*ncp->bprev = ncp->bnext) ){
       ncp->bnext->bprev = ncp->bprev;
     }
+  }else if(ncp->bnext){
+    ncp->bnext->bprev = NULL;
   }
   // recursively reparent our children to the plane to which we are bound.
   // this will extract each one from the sibling list.
   struct ncplane* bound = ncp->blist;
   while(bound){
     struct ncplane* tmp = bound->bnext;
-    if(ncplane_reparent_family(bound, ncp->boundto) == NULL){
+    ncplane* bindto = ((ncp == ncp->boundto) ? bound : ncp->boundto);
+    if(ncplane_reparent_family(bound, bindto) == NULL){
       ret = -1;
     }
     bound = tmp;
@@ -1000,6 +984,11 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   if(opts->flags >= (NCOPTION_DRAIN_INPUT << 1u)){
     fprintf(stderr, "Warning: unknown Notcurses options %016" PRIu64 "\n", opts->flags);
   }
+  if(opts->termtype){
+    if(putenv_term(opts->termtype)){
+      return NULL;
+    }
+  }
   notcurses* ret = malloc(sizeof(*ret));
   if(ret == NULL){
     return ret;
@@ -1091,6 +1080,7 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
                           opts->flags & NCOPTION_NO_FONT_CHANGES,
                           cursory, cursorx, &ret->stats,
                           ret->margin_l, ret->margin_t,
+                          ret->margin_r, ret->margin_b,
                           opts->flags & NCOPTION_DRAIN_INPUT)){
     fbuf_free(&ret->rstate.f);
     pthread_mutex_destroy(&ret->pilelock);
@@ -1121,19 +1111,17 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   }
   reset_term_attributes(&ret->tcache, &ret->rstate.f);
   const char* cinvis = get_escape(&ret->tcache, ESCAPE_CIVIS);
-  if(cinvis && term_emit(cinvis, ret->ttyfp, false)){
+  if(cinvis && fbuf_emit(&ret->rstate.f, cinvis) < 0){
     free_plane(ret->stdplane);
     goto err;
   }
   const char* pushcolors = get_escape(&ret->tcache, ESCAPE_SAVECOLORS);
-  if(pushcolors && term_emit(pushcolors, ret->ttyfp, false)){
+  if(pushcolors && fbuf_emit(&ret->rstate.f, pushcolors)){
     free_plane(ret->stdplane);
     goto err;
   }
   init_banner(ret, &ret->rstate.f);
-  fwrite(ret->rstate.f.buf, ret->rstate.f.used, 1, ret->ttyfp);
-  fbuf_reset(&ret->rstate.f);
-  if(ncflush(ret->ttyfp)){
+  if(fbuf_flush(&ret->rstate.f, ret->ttyfp) < 0){
     free_plane(ret->stdplane);
     goto err;
   }
@@ -1166,9 +1154,7 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
     }
   }
   if(ret->rstate.f.used){
-    fwrite(ret->rstate.f.buf, ret->rstate.f.used, 1, ret->ttyfp);
-    fbuf_reset(&ret->rstate.f);
-    if(ncflush(ret->ttyfp)){
+    if(fbuf_flush(&ret->rstate.f, ret->ttyfp) < 0){
       goto err;
     }
   }
@@ -1226,6 +1212,8 @@ void notcurses_drop_planes(notcurses* nc){
 }
 
 int notcurses_stop(notcurses* nc){
+  logdebug("stopping notcurses\n");
+//notcurses_debug(nc, stderr);
   int ret = 0;
   if(nc){
     ret |= notcurses_stop_minimal(nc);
@@ -1234,11 +1222,11 @@ int notcurses_stop(notcurses* nc){
     // PRESERVE_CURSOR was used, which is a bit more complex.
     if((nc->flags & NCOPTION_PRESERVE_CURSOR) || !get_escape(&nc->tcache, ESCAPE_SMCUP)){
       int targy = nc->rstate.logendy;
+      fbuf_reset(&nc->rstate.f);
       if(++targy >= nc->lfdimy){
-        printf("\n");
+        fbuf_putc(&nc->rstate.f, '\n');
         --targy;
       }
-      fbuf_reset(&nc->rstate.f);
       goto_location(nc, &nc->rstate.f, targy, 0);
       fbuf_finalize(&nc->rstate.f, stdout);
     }
@@ -1367,6 +1355,10 @@ int ncplane_move_above(ncplane* restrict n, ncplane* restrict above){
   if(n == above){
     return -1;
   }
+  if(above == NULL){
+    ncplane_move_bottom(n);
+    return 0;
+  }
   if(ncplane_pile(n) != ncplane_pile(above)){ // can't move among piles
     return -1;
   }
@@ -1398,6 +1390,10 @@ int ncplane_move_below(ncplane* restrict n, ncplane* restrict below){
   if(n == below){
     return -1;
   }
+  if(below == NULL){
+    ncplane_move_top(n);
+    return 0;
+  }
   if(ncplane_pile(n) != ncplane_pile(below)){ // can't move among piles
     return -1;
   }
@@ -1453,6 +1449,74 @@ void ncplane_move_bottom(ncplane* n){
   }
 }
 
+// if above is NULL, we're moving to the bottom
+int ncplane_move_family_above(ncplane* restrict n, ncplane* restrict bpoint){
+  ncplane* above = ncplane_above(n);
+  ncplane* below = ncplane_below(n);
+  if(ncplane_move_above(n, bpoint)){
+    return -1;
+  }
+  // traverse the planes above n, until we hit NULL. do the planes above n
+  // first, so that we know the topmost element of our new ensplicification.
+  // at this point, n is the bottommost plane, and we're inserting above it.
+  ncplane* targ = n;
+  while(above){
+    ncplane* tmp = ncplane_above(above);
+    if(ncplane_descendant_p(above, n)){
+      ncplane_move_above(above, targ);
+      targ = above;
+    }
+    above = tmp;
+  }
+  // n remains the topmost plane, and we're inserting above it. we have to be
+  // careful this time not to cross into any we moved below n.
+  const ncplane* topmost = targ;
+  targ = n;
+  while(below && below != topmost){
+    ncplane* tmp = ncplane_below(below);
+    if(ncplane_descendant_p(below, n)){
+      ncplane_move_below(below, targ);
+      targ = below;
+    }
+    below = tmp;
+  }
+  return 0;
+}
+
+// if below is NULL, we're moving to the top
+int ncplane_move_family_below(ncplane* restrict n, ncplane* restrict bpoint){
+  ncplane* below = ncplane_below(n);
+  ncplane* above = ncplane_above(n);
+  if(ncplane_move_below(n, bpoint)){
+    return -1;
+  }
+  // traverse the planes below n, until we hit NULL. do the planes below n
+  // first, so that we know the bottommost element of our new ensplicification.
+  // we're inserting below n...
+  ncplane* targ = n;
+  while(below){
+    ncplane* tmp = ncplane_below(below);
+    if(ncplane_descendant_p(below, n)){
+      ncplane_move_below(below, targ);
+      targ = below;
+    }
+    below = tmp;
+  }
+  // n remains the topmost plane, and we're inserting above it. we have to be
+  // careful this time not to cross into any we moved below n.
+  const ncplane* bottommost = targ;
+  targ = n;
+  while(above && above != bottommost){
+    ncplane* tmp = ncplane_above(above);
+    if(ncplane_descendant_p(above, n)){
+      ncplane_move_above(above, targ);
+      targ = above;
+    }
+    above = tmp;
+  }
+  return 0;
+}
+
 void ncplane_cursor_yx(const ncplane* n, int* y, int* x){
   if(y){
     *y = n->y;
@@ -2103,6 +2167,7 @@ void ncplane_yx(const ncplane* n, int* y, int* x){
 }
 
 void ncplane_erase(ncplane* n){
+  loginfo("erasing plane\n");
   if(n->sprite){
     sprixel_hide(n->sprite);
   }
@@ -2122,28 +2187,57 @@ void ncplane_erase(ncplane* n){
 }
 
 int ncplane_erase_region(ncplane* n, int ystart, int xstart, int ylen, int xlen){
-  if(ylen < 0 || xlen < 0){
-    logerror("Won't erase section of negative length (%d, %d)\n", ylen, xlen);
-    return -1;
+  if(ystart == -1){
+    ystart = n->y;
+  }
+  if(xstart == -1){
+    xstart = n->x;
   }
   if(ystart < 0 || xstart < 0){
     logerror("Illegal start of erase (%d, %d)\n", ystart, xstart);
     return -1;
   }
-  if(ystart >= ncplane_dim_y(n) || ystart + ylen > ncplane_dim_y(n)){
-    logerror("Illegal y spec for erase (%d, %d)\n", ystart, ylen);
+  if(ystart >= ncplane_dim_y(n) || xstart >= ncplane_dim_x(n)){
+    logerror("Illegal start of erase (%d, %d)\n", ystart, xstart);
     return -1;
   }
-  if(ylen == 0){
+  if(xlen < 0){
+    if(xlen + 1 < -xstart){
+      xlen = -xstart - 1;
+    }
+    xstart = xstart + xlen + 1;
+    xlen = -xlen;
+  }else if(xlen == 0){
+    xstart = 0;
+    xlen = ncplane_dim_x(n);
+  }
+  if(xlen > ncplane_dim_x(n) || xstart + xlen > ncplane_dim_x(n)){
+    xlen = ncplane_dim_x(n) - xstart;
+  }
+  if(ylen < 0){
+    if(ylen + 1 < -ystart){
+      ylen = -ystart - 1;
+    }
+    ystart = ystart + ylen + 1;
+    ylen = -ylen;
+  }else if(ylen == 0){
+    ystart = 0;
+    ylen = ncplane_dim_y(n);
+  }
+  if(ylen > ncplane_dim_y(n) || ystart + ylen > ncplane_dim_y(n)){
     ylen = ncplane_dim_y(n) - ystart;
   }
-  if(xstart >= ncplane_dim_x(n) || xstart + xlen > ncplane_dim_x(n)){
-    logerror("Illegal x spec for erase (%d, %d)\n", xstart, xlen);
-    return -1;
-  }
-  if(xlen == 0){
-    xlen = ncplane_dim_x(n) - ystart;
+  // special-case the full plane erasure, as it's powerfully optimized (O(1))
+  if(ystart == 0 && xstart == 0 &&
+      ylen == ncplane_dim_y(n) && xlen == ncplane_dim_x(n)){
+    int tmpy = n->y; // preserve cursor location
+    int tmpx = n->x;
+    ncplane_erase(n);
+    n->y = tmpy;
+    n->x = tmpx;
+    return 0;
   }
+  loginfo("erasing %d/%d - %d/%d\n", ystart, xstart, ystart + ylen, xstart + xlen);
   for(int y = ystart ; y < ystart + ylen ; ++y){
     for(int x = xstart ; x < xstart + xlen ; ++x){
       nccell_release(n, &n->fb[nfbcellidx(n, y, x)]);
@@ -2335,8 +2429,18 @@ const ncplane* ncplane_parent_const(const ncplane* n){
   return n->boundto;
 }
 
-ncplane* ncplane_boundlist(ncplane* n){
-  return n->blist;
+int ncplane_set_name(ncplane* n, const char* name){
+  char* copy = name ? strdup(name) : NULL;
+  if(copy == NULL && name != NULL){
+    return -1;
+  }
+  free(n->name);
+  n->name = copy;
+  return 0;
+}
+
+char* ncplane_name(const ncplane* n){
+  return n->name ? strdup(n->name) : NULL;
 }
 
 void ncplane_set_resizecb(ncplane* n, int(*resizecb)(ncplane*)){
@@ -2459,19 +2563,21 @@ ncplane* ncplane_reparent(ncplane* n, ncplane* newparent){
 // to be called before unbinding 'n' from old pile.
 static void
 unsplice_zaxis_recursive(ncplane* n){
+  // might already have been unspliced, in which case ->above/->below are NULL
   if(ncplane_pile(n)->top == n){
     ncplane_pile(n)->top = n->below;
-  }else{
+  }else if(n->above){
     n->above->below = n->below;
   }
   if(ncplane_pile(n)->bottom == n){
     ncplane_pile(n)->bottom = n->above;
-  }else{
+  }else if(n->below){
     n->below->above = n->above;
   }
   for(ncplane* child = n->blist ; child ; child = child->bnext){
     unsplice_zaxis_recursive(child);
   }
+  n->below = n->above = NULL;
 }
 
 // unsplice our sprixel from the pile's sprixellist, and then unsplice all
@@ -2497,7 +2603,7 @@ unsplice_sprixels_recursive(ncplane* n, sprixel* prev){
   }
   for(ncplane* child = n->blist ; child ; child = child->bnext){
     unsplice_sprixels_recursive(child, prev);
-    while(prev->next){ // FIXME lame
+    while(prev && prev->next){ // FIXME lame
       prev = prev->next;
     }
   }
@@ -2507,7 +2613,8 @@ unsplice_sprixels_recursive(ncplane* n, sprixel* prev){
 // recursively splice 'n' and children into the z-axis, above 'n->boundto'.
 // handles 'n' == 'n->boundto'. to be called after binding 'n' into new pile.
 static void
-splice_zaxis_recursive(ncplane* n){
+splice_zaxis_recursive(ncplane* n, ncpile* p){
+  n->pile = p;
   if(n != n->boundto){
     if((n->above = n->boundto->above) == NULL){
       n->pile->top = n;
@@ -2518,7 +2625,7 @@ splice_zaxis_recursive(ncplane* n){
     n->boundto->above = n;
   }
   for(ncplane* child = n->blist ; child ; child = child->bnext){
-    splice_zaxis_recursive(child);
+    splice_zaxis_recursive(child, p);
   }
 }
 
@@ -2537,7 +2644,11 @@ ncplane* ncplane_reparent_family(ncplane* n, ncplane* newparent){
     if( (*n->bprev = n->bnext) ){
       n->bnext->bprev = n->bprev;
     }
+  }else if(n->bnext){
+    n->bnext->bprev = NULL;
   }
+  n->bprev = NULL;
+  n->bnext = NULL;
   // ncplane_notcurses() goes through ncplane_pile(). since we're possibly
   // destroying piles below, get the notcurses reference early on.
   notcurses* nc = ncplane_notcurses(n);
@@ -2549,15 +2660,17 @@ ncplane* ncplane_reparent_family(ncplane* n, ncplane* newparent){
   }
   n->boundto = newparent;
   if(n == n->boundto){ // we're a new root plane
+    logdebug("reparenting new root plane %p\n", n);
+    unsplice_zaxis_recursive(n);
     n->bnext = NULL;
     n->bprev = NULL;
-    splice_zaxis_recursive(n);
     pthread_mutex_lock(&nc->pilelock);
     if(ncplane_pile(n)->top == NULL){ // did we just empty our pile?
       ncpile_destroy(ncplane_pile(n));
     }
     make_ncpile(ncplane_notcurses(n), n);
     pthread_mutex_unlock(&nc->pilelock);
+    splice_zaxis_recursive(n, ncplane_pile(n));
   }else{ // establish ourselves as a sibling of new parent's children
     if( (n->bnext = newparent->blist) ){
       n->bnext->bprev = &n->bnext;
@@ -2566,13 +2679,13 @@ ncplane* ncplane_reparent_family(ncplane* n, ncplane* newparent){
     newparent->blist = n;
     // place it immediately above the new binding plane if crossing piles
     if(n->pile != ncplane_pile(n->boundto)){
-      splice_zaxis_recursive(n);
       pthread_mutex_lock(&nc->pilelock);
       if(ncplane_pile(n)->top == NULL){ // did we just empty our pile?
         ncpile_destroy(ncplane_pile(n));
       }
       n->pile = ncplane_pile(n->boundto);
       pthread_mutex_unlock(&nc->pilelock);
+      splice_zaxis_recursive(n, ncplane_pile(n));
     }
   }
   if(s){ // must be on new plane, with sprixels to donate
