@@ -41,6 +41,10 @@ create_pixelp(ncplot *p, ncplane* n){
   if(((p->pixelp = ncplane_dup(n, NULL)) == NULL)){
     return -1;
   }
+  if(ncplane_set_name(p->pixelp, "pmap")){
+    ncplane_destroy(p->pixelp);
+    return -1;
+  }
   ncplane_reparent(p->pixelp, n);
   ncplane_move_below(p->pixelp, n);
   uint64_t basechan = 0;
@@ -409,7 +413,7 @@ create_##T(nc##X##plot* ncpp, ncplane* n, const ncplot_options* opts, const T mi
     opts = &zeroed; \
   } \
   if(opts->flags >= (NCPLOT_OPTION_PRINTSAMPLE << 1u)){ \
-    logwarn("Provided unsupported flags %016jx\n", (uintmax_t)opts->flags); \
+    logwarn("Provided unsupported flags %016" PRIx64 "\n", opts->flags); \
   } \
   /* if miny == maxy (enabling domain detection), they both must be equal to 0 */ \
   if(miny == maxy && miny){ \
@@ -510,9 +514,7 @@ create_##T(nc##X##plot* ncpp, ncplane* n, const ncplot_options* opts, const T mi
    reset them, and write the new sample anywhere. otherwise, write it to the \
    proper slot based on the current newest slot. */ \
 int window_slide_##T(nc##X##plot* ncp, int64_t x){ \
-  if(x < ncp->plot.slotx - (ncp->plot.slotcount - 1)){ /* x is behind window, won't be counted */ \
-    return -1; \
-  }else if(x <= ncp->plot.slotx){ /* x is within window, do nothing */ \
+  if(x <= ncp->plot.slotx){ /* x is within window, do nothing */ \
     return 0; \
   } /* x is newest; we might be keeping some, might not */ \
   int64_t xdiff = x - ncp->plot.slotx; /* the raw amount we're advancing */ \
@@ -547,7 +549,13 @@ static void update_sample_##T(nc##X##plot* ncp, int64_t x, T y, bool reset); \
    x window, the x window is advanced to include x, and values passing beyond \
    the window are lost. The first call will place the initial window. The plot \
    will be redrawn, but notcurses_render() is not called. */ \
-int add_sample_##T(nc##X##plot* ncpp, uint64_t x, T y){ \
+int add_sample_##T(nc##X##plot* ncpp, int64_t x, T y){ \
+  if(x < ncpp->plot.slotx - (ncpp->plot.slotcount - 1)){ /* x is behind window, won't be counted */ \
+    return -1; \
+  } \
+  if(y == 0 && x <= ncpp->plot.slotx){ \
+    return 0; /* no need to redraw plot; nothing changed */ \
+  } \
   if(window_slide_##T(ncpp, x)){ \
     return -1; \
   } \
