@@ -146,6 +146,7 @@ static uint32_t
 highcontrast(const tinfo* ti, uint32_t bchannel){
   unsigned r, g, b;
   if(ncchannel_default_p(bchannel)){
+    // FIXME what if we couldn't identify the background color?
     r = ncchannel_r(ti->bg_collides_default);
     g = ncchannel_g(ti->bg_collides_default);
     b = ncchannel_b(ti->bg_collides_default);
@@ -691,15 +692,13 @@ term_bg_rgb8(const tinfo* ti, fbuf* f, unsigned r, unsigned g, unsigned b){
   // we're also in that case working with hopefully more robust terminals.
   // If it doesn't work, eh, it doesn't work. Fuck the world; save yourself.
   if(ti->caps.rgb){
-    if(ti->bg_collides_default){
-      if((r == (ti->bg_collides_default & 0xff0000lu)) &&
-         (g == (ti->bg_collides_default & 0xff00lu)) &&
-         (b == (ti->bg_collides_default & 0xfflu))){
-        if(b < 255){
-          ++b;
-        }else{
-          --b;
-        }
+    if((ti->bg_collides_default & 0xff000000) == 0x01000000){
+      if((r == ncchannel_r(ti->bg_collides_default)) &&
+         (g == ncchannel_g(ti->bg_collides_default)) &&
+         (b == ncchannel_b(ti->bg_collides_default))){
+        // the human eye has fewer blue cones than red or green. toggle
+        // the last bit in the blue component to avoid a collision.
+        b ^= 0x00000001;
       }
     }
     return term_esc_rgb(f, false, r, g, b);
@@ -1496,7 +1495,7 @@ engorge_crender_vector(ncpile* n){
   const size_t crenderlen = n->dimy * n->dimx; // desired size
 //fprintf(stderr, "crlen: %d y: %d x:%d\n", crenderlen, dimy, dimx);
   if(crenderlen != n->crenderlen){
-    loginfo("Resizing rvec (%zu) for %p to %zu\n", n->crenderlen, n, crenderlen);
+    loginfo("Resizing rvec (%lu) for %p to %lu\n", (long unsigned)n->crenderlen, n, (long unsigned)crenderlen);
     struct crender* tmp = realloc(n->crender, sizeof(*tmp) * crenderlen);
     if(tmp == NULL){
       return -1;
