@@ -51,7 +51,7 @@ void sprixel_free(sprixel* s){
 sprixel* sprixel_recycle(ncplane* n){
   assert(n->sprite);
   const notcurses* nc = ncplane_notcurses_const(n);
-  if(nc->tcache.pixel_shutdown == kitty_shutdown){
+  if(nc->tcache.pixel_implementation >= NCPIXEL_KITTY_STATIC){
     sprixel* hides = n->sprite;
     int dimy = hides->dimy;
     int dimx = hides->dimx;
@@ -220,7 +220,7 @@ int sprite_init(const tinfo* t, int fd){
   if(t->pixel_init == NULL){
     return 0;
   }
-  return t->pixel_init(t, fd);
+  return t->pixel_init(fd);
 }
 
 uint8_t* sprixel_auxiliary_vector(const sprixel* s){
