@@ -41,9 +41,13 @@ get_default_geometry(tinfo* ti){
   loginfo("Default geometry: %d row%s, %d column%s\n",
           ti->default_rows, ti->default_rows != 1 ? "s" : "",
           ti->default_cols, ti->default_cols != 1 ? "s" : "");
+  ti->dimy = ti->default_rows;
+  ti->dimx = ti->default_cols;
 }
 
-// we found Sixel support -- set up the API
+// we found Sixel support -- set up its API. invert80 refers to whether the
+// terminal implements DECSDM correctly (enabling it with \e[?80h), or inverts
+// the meaning (*disabling* it with \e[?80h) (we always want it disabled).
 static inline void
 setup_sixel_bitmaps(tinfo* ti, int fd, bool invert80){
   if(invert80){
@@ -60,7 +64,6 @@ setup_sixel_bitmaps(tinfo* ti, int fd, bool invert80){
   ti->pixel_move = NULL;
   ti->pixel_scroll = NULL;
   ti->pixel_wipe = sixel_wipe;
-  ti->pixel_shutdown = sixel_shutdown;
   ti->pixel_clear_all = NULL;
   ti->pixel_rebuild = sixel_rebuild;
   ti->pixel_trans_auxvec = sixel_trans_auxvec;
@@ -83,7 +86,6 @@ setup_kitty_bitmaps(tinfo* ti, int fd, ncpixelimpl_e level){
   ti->pixel_commit = kitty_commit;
   ti->pixel_move = kitty_move;
   ti->pixel_scroll = NULL;
-  ti->pixel_shutdown = kitty_shutdown;
   ti->pixel_clear_all = kitty_clear_all;
   if(level == NCPIXEL_KITTY_STATIC){
     ti->pixel_wipe = kitty_wipe;
@@ -121,7 +123,6 @@ setup_fbcon_bitmaps(tinfo* ti, int fd){
   ti->pixel_refresh = NULL;
   ti->pixel_move = NULL;
   ti->pixel_scroll = fbcon_scroll;
-  ti->pixel_shutdown = NULL;
   ti->pixel_clear_all = NULL;
   ti->pixel_rebuild = fbcon_rebuild;
   ti->pixel_wipe = fbcon_wipe;
@@ -153,18 +154,6 @@ query_rgb(void){
   return rgb;
 }
 
-// we couldn't get a terminal from interrogation, so let's see if the TERM
-// matches any of our known terminals. this can only be as accurate as the
-// TERM setting is (and as up-to-date and complete as we are).
-static int
-match_termname(const char* termname, queried_terminals_e* qterm){
-  // https://github.com/alacritty/alacritty/pull/5274 le sigh
-  if(strstr(termname, "alacritty")){
-    *qterm = TERMINAL_ALACRITTY;
-  }
-  return 0;
-}
-
 void free_terminfo_cache(tinfo* ti){
   stop_inputlayer(ti);
   free(ti->termversion);
@@ -190,35 +179,40 @@ compare_versions(const char* restrict v1, const char* restrict v2){
   if(v1 == NULL){
     return -1;
   }
+  char* v1e;
+  char* v2e;
   while(*v1 && *v2){
-    if(isdigit(*v1) && isdigit(*v2)){
-      if(*v1 > *v2){
-        return 1;
-      }
-      if(*v2 > *v1){
-        return -1;
-      }
-    }else if(isdigit(*v1)){
-      return 1;
-    }else if(isdigit(*v2)){
+    long v1v = strtol(v1, &v1e, 10);
+    long v2v = strtol(v2, &v2e, 10);
+    if(v1e == v1 && v2e == v2){ // both are done
+      return 0;
+    }else if(v1e == v1){ // first is done
       return -1;
-    }else if(*v1 != '.' && *v2 != '.'){
+    }else if(v2e == v2){ // second is done
+      return 1;
+    }
+    if(v1v > v2v){
+      return 1;
+    }else if(v2v > v1v){
+      return -1;
+    }
+    if(*v1e != '.' && *v2e != '.'){
       break;
-    }else if(*v1 != '.' || *v2 != '.'){
-      if(*v1 == '.'){
+    }else if(*v1e != '.' || *v2e != '.'){
+      if(*v1e == '.'){
         return 1;
       }else{
         return -1;
       }
     }
-    ++v1;
-    ++v2;
+    v1 = v1e + 1;
+    v2 = v2e + 1;
   }
   // can only get out here if at least one was not a period
-  if(*v1 == '.'){
+  if(*v1e == '.'){
     return 1;
   }
-  if(*v2 == '.'){
+  if(*v2e == '.'){
     return -1;
   }
   return 0;
@@ -324,21 +318,25 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 // which can be identified directly, sans queries.
 #define KITTYQUERY "\x1b_Gi=1,a=q;\x1b\\"
 
-// request kitty keyboard protocol through level 1, and push current.
+// request kitty keyboard protocol through level 1, first pushing current.
 // see https://sw.kovidgoyal.net/kitty/keyboard-protocol/#progressive-enhancement
-#define KBDSUPPORT "\x1b[>u\x1b[=1u"
+// FIXME go to level 11, but first need handle all functional keys
+#define KBDSUPPORT "\x1b[=3u"
 
 // the kitty keyboard protocol allows unambiguous, complete identification of
-// input events. this queries for the level of support.
+// input events. this queries for the level of support. we want to do this
+// because the "keyboard pop" control code is mishandled by kitty < 0.20.0.
 #define KBDQUERY "\x1b[?u"
 
-// these queries (terminated with a Primary Device Attributes, to which
-// all known terminals reply) hopefully can uniquely and unquestionably
-// identify the terminal to which we are talking.
-#define IDQUERIES KITTYQUERY \
-                  KBDSUPPORT \
-                  KBDQUERY \
-                  TRIDEVATTR \
+// set modifyFunctionKeys (2) if supported, allowing us to disambiguate
+// function keys when used with modifiers. set modifyOtherKeys (4) if
+// supported. these ought follow keyboard push and precede kitty keyboard.
+#define XTMODKEYS "\x1b[>2;1m\x1b[>4;1m"
+
+// these queries can hopefully uniquely and unquestionably identify the
+// terminal to which we are talking. if we already know what we're talking
+// to, there's no point in sending them.
+#define IDQUERIES TRIDEVATTR \
                   XTVERSION \
                   XTGETTCAPTN \
                   SECDEVATTR
@@ -373,6 +371,11 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 #define DIRECTIVES CSI_BGQ \
                    SUMQUERY \
                    "\x1b[?1;3;256S" /* try to set 256 cregs */ \
+                   KKEYBOARD_PUSH \
+                   XTMODKEYS \
+                   KBDSUPPORT \
+                   KBDQUERY \
+                   KITTYQUERY \
                    CREGSXTSM \
                    GEOMXTSM \
                    GEOMPIXEL \
@@ -383,6 +386,7 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 // terminfo, but everyone who supports it supports it the same way, and we
 // need to send it before our other directives if we're going to use it.
 #define SMCUP "\x1b[?1049h"
+#define RMCUP "\x1b[?1049l"
 
 // we send an XTSMGRAPHICS to set up 256 color registers (the most we can
 // currently take advantage of; we need at least 64 to use sixel at all).
@@ -502,15 +506,6 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
     // setupterm interprets a missing/empty TERM variable as the special value “unknown”.
     termname = ti->termname ? ti->termname : "unknown";
   }
-  if(qterm == TERMINAL_UNKNOWN){
-    match_termname(termname, &qterm);
-    // we pick up alacritty's version via a weird hack involving Secondary
-    // Device Attributes. if we're not alacritty, don't trust that version.
-    if(qterm != TERMINAL_ALACRITTY){
-      free(ti->termversion);
-      ti->termversion = NULL;
-    }
-  }
   // st had neither caps.sextants nor caps.quadrants last i checked (0.8.4)
   ti->caps.braille = true; // most everyone has working caps.braille, even from fonts
   ti->caps.halfblocks = true; // most everyone has working halfblocks
@@ -547,6 +542,9 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
     if(add_appsync_escapes_dcs(ti, tablelen, tableused)){
       return -1;
     }
+    if(compare_versions(ti->termversion, "0.15.1") < 0){
+      *invertsixel = true;
+    }
   }else if(qterm == TERMINAL_VTE){
     termname = "VTE";
     ti->caps.quadrants = true;
@@ -560,7 +558,7 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
     ti->caps.sextants = true;
     ti->caps.quadrants = true;
     ti->caps.rgb = true;
-    if(compare_versions(ti->termversion, "1.8.2") >= 0){
+    if(compare_versions(ti->termversion, "1.8.2") < 0){
       *invertsixel = true;
     }
   }else if(qterm == TERMINAL_TMUX){
@@ -569,11 +567,6 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
   }else if(qterm == TERMINAL_MLTERM){
     termname = "MLterm";
     ti->caps.quadrants = true; // good caps.quadrants, no caps.sextants as of 3.9.0
-    // MLterm 3.9.1 brings it into line with other terminals, and we
-    // stop inverting the meaning / applying the cursor hack for 3.9.1+.
-    if(compare_versions(ti->termversion, "3.9.1") < 0){
-      *invertsixel = true;
-    }
   }else if(qterm == TERMINAL_WEZTERM){
     termname = "WezTerm";
     ti->caps.rgb = true;
@@ -591,17 +584,24 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
       if(add_pushcolors_escapes(ti, tablelen, tableused)){
         return -1;
       }
+      if(compare_versions(ti->termversion, "359") < 0){
+        *invertsixel = true;
+      }
     }
   }else if(qterm == TERMINAL_MINTTY){
     termname = "MinTTY";
     if(add_smulx_escapes(ti, tablelen, tableused)){
       return -1;
     }
+    if(compare_versions(ti->termversion, "3.5.2") < 0){
+      *invertsixel = true;
+    }
     ti->bce = true;
   }else if(qterm == TERMINAL_CONTOUR){
     termname = "Contour";
     ti->caps.quadrants = true;
     ti->caps.rgb = true;
+    *invertsixel = true;
   }else if(qterm == TERMINAL_ITERM){
     termname = "iTerm2";
     // iTerm implements DCS ASU, but has no detection for it
@@ -720,7 +720,8 @@ macos_early_matches(void){
 int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned utf8,
                          unsigned noaltscreen, unsigned nocbreak, unsigned nonewfonts,
                          int* cursor_y, int* cursor_x, ncsharedstats* stats,
-                         int lmargin, int tmargin, unsigned draininput){
+                         int lmargin, int tmargin, int rmargin, int bmargin,
+                         unsigned draininput){
   int foolcursor_x, foolcursor_y;
   if(!cursor_x){
     cursor_x = &foolcursor_x;
@@ -730,6 +731,7 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
   }
   *cursor_x = *cursor_y = -1;
   memset(ti, 0, sizeof(*ti));
+  ti->bg_collides_default = 0xfe000000;
   ti->qterm = TERMINAL_UNKNOWN;
   // we don't need a controlling tty for everything we do; allow a failure here
   ti->ttyfd = get_tty_fd(out);
@@ -772,7 +774,6 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
       free(ti->tpreserved);
       return -1;
     }
-// FIXME need to enter alternate screen here
     // if we already know our terminal (e.g. on the linux console), there's no
     // need to send the identification queries. the controls are sufficient.
     bool minimal = (ti->qterm != TERMINAL_UNKNOWN);
@@ -791,7 +792,14 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
   }
   tname = termname(); // longname() is also available
 #endif
-  if(init_inputlayer(ti, stdin, lmargin, tmargin, stats, draininput)){
+  int linesigs_enabled = 1;
+  if(ti->tpreserved){
+    if(!(ti->tpreserved->c_lflag & ISIG)){
+      linesigs_enabled = 0;
+    }
+  }
+  if(init_inputlayer(ti, stdin, lmargin, tmargin, rmargin, bmargin,
+                     stats, draininput, linesigs_enabled)){
     goto err;
   }
   ti->sprixel_scale_height = 1;
@@ -933,6 +941,7 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
         goto err;
       }
     }
+    ti->kbdlevel = iresp->kbdlevel;
     if(iresp->qterm != TERMINAL_UNKNOWN){
       ti->qterm = iresp->qterm;
     }
@@ -944,6 +953,8 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
       // random transient measurement?
       ti->default_rows = iresp->dimy;
       ti->default_cols = iresp->dimx;
+      ti->dimy = iresp->dimy;
+      ti->dimx = iresp->dimx;
     }
     if(iresp->pixy && iresp->pixx){
       ti->pixy = iresp->pixy;
@@ -953,7 +964,11 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
       ti->cellpixy = ti->pixy / ti->default_rows;
       ti->cellpixx = ti->pixx / ti->default_cols;
     }
-    ti->bg_collides_default = iresp->bg;
+    if(iresp->got_bg){
+      // reset the 0xfe000000 we loaded during initialization. if we're
+      // kitty, we'll add the 0x01000000 in during heuristics.
+      ti->bg_collides_default = iresp->bg;
+    }
     // kitty trumps sixel, when both are available
     if((kitty_graphics = iresp->kitty_graphics) == 0){
       ti->color_registers = iresp->color_registers;
@@ -961,9 +976,7 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
       ti->sixel_maxx = iresp->sixelx;
     }
     free(iresp);
-  }
-  if(nocbreak){
-    if(ti->ttyfd >= 0){
+    if(nocbreak){
       // FIXME do this in input later, upon signaling completion?
       if(tcsetattr(ti->ttyfd, TCSANOW, ti->tpreserved)){
         goto err;
@@ -983,7 +996,7 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
   build_supported_styles(ti);
   if(ti->pixel_draw == NULL && ti->pixel_draw_late == NULL){
     if(kitty_graphics){
-      setup_kitty_bitmaps(ti, ti->ttyfd, NCPIXEL_KITTY_ANIMATED);
+      setup_kitty_bitmaps(ti, ti->ttyfd, NCPIXEL_KITTY_STATIC);
     }
     // our current sixel quantization algorithm requires at least 64 color
     // registers. we make use of no more than 256. this needs to happen
@@ -995,7 +1008,10 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out, unsigned ut
   return 0;
 
 err:
-  // FIXME need to leave alternate screen if we entered it
+  if(ti->ttyfd >= 0){
+    tty_emit(KKEYBOARD_POP, ti->ttyfd);
+    tty_emit(RMCUP, ti->ttyfd);
+  }
   if(ti->tpreserved){
     (void)tcsetattr(ti->ttyfd, TCSANOW, ti->tpreserved);
     free(ti->tpreserved);
@@ -1056,14 +1072,32 @@ int locate_cursor(tinfo* ti, int* cursor_y, int* cursor_x){
     return -1;
   }
   int fd = ti->ttyfd;
-  if(tty_emit(u7, fd)){
+  if(get_cursor_location(ti->ictx, u7, cursor_y, cursor_x)){
     return -1;
   }
-  get_cursor_location(ti->ictx, cursor_y, cursor_x);
   loginfo("got a report from %d %d/%d\n", fd, *cursor_y, *cursor_x);
   return 0;
 }
 
+int tiocgwinsz(int fd, struct winsize* ws){
+#ifndef __MINGW64__
+  int i = ioctl(fd, TIOCGWINSZ, ws);
+  if(i < 0){
+    logerror("TIOCGWINSZ failed on %d (%s)\n", fd, strerror(errno));
+    return -1;
+  }
+  if(ws->ws_row <= 0 || ws->ws_col <= 0){
+    logerror("Bogus return from TIOCGWINSZ on %d (%d/%d)\n",
+             fd, ws->ws_row, ws->ws_col);
+    return -1;
+  }
+#else
+  (void)fd;
+  (void)ws;
+#endif
+  return 0;
+}
+
 int cbreak_mode(tinfo* ti){
 #ifndef __MINGW64__
   int ttyfd = ti->ttyfd;
@@ -1102,3 +1136,27 @@ int cbreak_mode(tinfo* ti){
 #endif
   return 0;
 }
+
+// replace or populate the TERM environment variable with 'termname'
+int putenv_term(const char* termname){
+  #define ENVVAR "TERM"
+  const char* oldterm = getenv(ENVVAR);
+  if(oldterm){
+    logdebug("replacing %s value %s with %s\n", ENVVAR, oldterm, termname);
+  }else{
+    loginfo("provided %s value %s\n", ENVVAR, termname);
+  }
+  if(strcmp(oldterm, termname) == 0){
+    return 0;
+  }
+  char* buf = malloc(strlen(termname) + strlen(ENVVAR) + 1);
+  if(buf == NULL){
+    return -1;
+  }
+  int c = putenv(buf);
+  if(c){
+    logerror("couldn't export %s\n", buf);
+  }
+  free(buf);
+  return c;
+}
