@@ -16,6 +16,9 @@ extern "C" {
 #include "fbuf.h"
 #include "in.h"
 
+#define KKEYBOARD_POP  "\x1b[<u"
+#define KKEYBOARD_PUSH "\x1b[>u"
+
 struct ncpile;
 struct sprixel;
 struct notcurses;
@@ -104,13 +107,15 @@ typedef struct tinfo {
   // be acquired on all terminals with pixel support.
   int cellpixy;                    // cell pixel height, might be 0
   int cellpixx;                    // cell pixel width, might be 0
+  int dimy, dimx;                  // most recent cell geometry
 
   unsigned supported_styles; // bitmask over NCSTYLE_* driven via sgr/ncv
 
   // kitty interprets an RGB background that matches the default background
   // color *as* the default background, meaning it'll be translucent if
   // background_opaque is in use. detect this, and avoid the default if so.
-  // bg_collides_default is either 0x0000000 or (if in use) 0x1RRGGBB.
+  // bg_collides_default is either:
+  // 0xfexxxxxxx (unknown), 0x00RRGGBB (no collide), or 0x01RRGGBB (collides).
   uint32_t bg_collides_default;
 
   // bitmap support. if we support bitmaps, pixel_implementation will be a
@@ -127,14 +132,13 @@ typedef struct tinfo {
   // it leaves the sprixel in INVALIDATED so that it's drawn in phase 2.
   void (*pixel_refresh)(const struct ncpile* p, struct sprixel* s);
   int (*pixel_remove)(int id, fbuf* f); // kitty only, issue actual delete command
-  int (*pixel_init)(const struct tinfo*, int fd); // called when support is detected
+  int (*pixel_init)(int fd); // called when support is detected
   int (*pixel_draw)(const struct tinfo*, const struct ncpile* p,
                     struct sprixel* s, fbuf* f, int y, int x);
   int (*pixel_draw_late)(const struct tinfo*, struct sprixel* s, int yoff, int xoff);
   // execute move (erase old graphic, place at new location) if non-NULL
   int (*pixel_move)(struct sprixel* s, fbuf* f, unsigned noscroll, int yoff, int xoff);
   int (*pixel_scrub)(const struct ncpile* p, struct sprixel* s);
-  int (*pixel_shutdown)(fbuf* f);   // called during context shutdown
   int (*pixel_clear_all)(fbuf* f);  // called during context startup
   // make a loaded graphic visible. only used with kitty.
   int (*pixel_commit)(fbuf* f, struct sprixel* s, unsigned noscroll);
@@ -182,10 +186,18 @@ typedef struct tinfo {
   HANDLE outhandle;
 #endif
 
+  unsigned kbdlevel;         // kitty keyboard support level
   bool bce;                  // is the bce property advertised?
   bool in_alt_screen;        // are we in the alternate screen?
 } tinfo;
 
+static inline void
+update_tinfo_geometry(tinfo* ti, int dimy, int dimx){
+  // FIXME probably need a lock on this!
+  ti->dimy = dimy;
+  ti->dimx = dimx;
+}
+
 // retrieve the terminfo(5)-style escape 'e' from tdesc (NULL if undefined).
 static inline __attribute__ ((pure)) const char*
 get_escape(const tinfo* tdesc, escape_e e){
@@ -211,7 +223,7 @@ int interrogate_terminfo(tinfo* ti, const char* termtype, FILE* out,
                          unsigned utf8, unsigned noaltscreen, unsigned nocbreak,
                          unsigned nonewfonts, int* cursor_y, int* cursor_x,
                          struct ncsharedstats* stats, int lmargin, int tmargin,
-                         unsigned draininput)
+                         int rmargin, int bmargin, unsigned draininput)
   __attribute__ ((nonnull (1, 3, 10)));
 
 void free_terminfo_cache(tinfo* ti);
@@ -302,7 +314,8 @@ term_emit(const char* seq, FILE* out, bool flush){
     return -1;
   }
   if(ncfputs(seq, out) == EOF){
-    logerror("Error emitting %zub escape (%s)\n", strlen(seq), strerror(errno));
+    logerror("Error emitting %lub escape (%s)\n",
+             (unsigned long)strlen(seq), strerror(errno));
     return -1;
   }
   return flush ? ncflush(out) : 0;
@@ -344,6 +357,9 @@ leave_alternate_screen(FILE* fp, tinfo* ti){
 
 int cbreak_mode(tinfo* ti);
 
+// execute termios's TIOCGWINSZ ioctl(). returns -1 on failure.
+int tiocgwinsz(int fd, struct winsize* ws);
+
 #ifdef __cplusplus
 }
 #endif
