@@ -9,6 +9,7 @@
 #include <unistd.h>
 #include <iostream>
 #include <inttypes.h>
+#include <ncpp/Input.hh>
 #include <ncpp/Direct.hh>
 #include <ncpp/Visual.hh>
 #include <ncpp/NotCurses.hh>
@@ -103,6 +104,11 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
     }else{
       keyp = nc.get(false, &ni);
     }
+    // we don't care about key release events, especially the enter
+    // release that starts so many interactive programs under Kitty
+    if(ni.evtype == EvType::Release){
+      continue;
+    }
     if(keyp == 0){
       break;
     }
@@ -112,7 +118,7 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
         return -1;
       }
     }
-    // if we just hit a non-space character to unpause, interpret it
+    // if we just hit a non-space character to unpause, ignore it
     if(keyp == ' '){ // space for unpause
       continue;
     }
