@@ -86,7 +86,7 @@ reset_terminal(){
       tios.c_lflag |= ISIG | ICANON | ECHO;
       tcsetattr(fd, TCSADRAIN, &tios);
     }
-    printf("\x1b[<u"); // pop any kitty keyboard state we set
+    printf(KKEYBOARD_POP);
     char* str = tigetstr("sgr0");
     if(str != (char*)-1){
       printf("%s", str);
@@ -134,14 +134,14 @@ auto lang_and_term() -> void {
     std::cerr << "TERM wasn't defined, exiting with success" << std::endl;
     exit(EXIT_SUCCESS);
   }
+  std::cout << "Running with TERM=" << term << std::endl;
   auto nc = testing_notcurses();
   if(!nc){
     exit(EXIT_FAILURE);
   }
   int dimy, dimx;
   notcurses_stddim_yx(nc, &dimy, &dimx);
-  std::cout << "Running with TERM=" << term << " (" << dimx << 'x'
-            << dimy << ")" << std::endl;
+  std::cout << "Detected geometry: " << dimx << 'x' << dimy << std::endl;
   notcurses_stop(nc);
   if(dimx < 50 || dimy < 24){ // minimum assumed geometry
     std::cerr << "Terminal was too small for tests (minimum 50x24)" << std::endl;
@@ -164,7 +164,9 @@ auto main(int argc, const char **argv) -> int {
   handle_opts(argv);
   int res = context.run(); // run
   reset_terminal();
-  check_data_dir();
+  if(res){
+    check_data_dir();
+  }
   if(context.shouldExit()){ // important - query flags (and --exit) rely on the user doing this
     return res;             // propagate the result of the tests
   }
