@@ -872,7 +872,8 @@ TEST_CASE("Plane") {
     struct ncplane* n = ncplane_create(n_, &nopts);
     REQUIRE(n);
     ncinput ni{};
-    ni.id = NCKEY_RELEASE;
+    ni.id = NCKEY_BUTTON1;
+    ni.evtype = ncinput::NCTYPE_RELEASE;
     int total = 0;
     for(ni.y = 0 ; ni.y < 5 ; ++ni.y){
       for(ni.x = 0 ; ni.x < 5 ; ++ni.x){
